#!/usr/bin/env python
# -*- coding: utf-8 -*-

# This work was created by participants in the DataONE project, and is
# jointly copyrighted by participating institutions in DataONE. For
# more information on DataONE, see our web site at http://dataone.org.
#
#   Copyright ${year}
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#   http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

'''
Module d1_common.tests.test_mime_multipart
==========================================

Unit tests for MIME Multipart document generator.

:Created: 2011-03-17
:Author: DataONE (Dahl)
:Dependencies:
  - python 2.6
'''

# Stdlib.
import logging
import sys
import unittest
import StringIO
import re

# D1.
from d1_common import xmlrunner
import d1_common.mime_multipart

# App
import util


sysmeta_xml_correct = u"""<?xml version="1.0" encoding="UTF-8"?>
<d1:systemMetadata xmlns:d1="http://ns.dataone.org/service/types/v1"
 xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
 xsi:schemaLocation="http://ns.dataone.org/service/types/v1">
    <identifier>identifier0</identifier>
    <objectFormat>
        <formatId>formatId0</formatId>
        <formatName>formatName0</formatName>
        <scienceMetadata>false</scienceMetadata>
    </objectFormat>
    <size>-1073741773</size>
    <checksum algorithm="SHA-1">checksum0</checksum>
    <submitter>submitter0</submitter>
    <rightsHolder>rightsHolder0</rightsHolder>
    <accessPolicy>
        <allow>
            <subject>subject0</subject>
            <subject>subject1</subject>
            <permission>read</permission>
            <permission>read</permission>
        </allow>
        <allow>
            <subject>subject2</subject>
            <subject>subject3</subject>
            <permission>read</permission>
            <permission>read</permission>
        </allow>
    </accessPolicy>
    <replicationPolicy replicationAllowed="false" numberReplicas="0">
        <preferredMemberNode>preferredMemberNode0</preferredMemberNode>
        <preferredMemberNode>preferredMemberNode1</preferredMemberNode>
        <blockedMemberNode>blockedMemberNode0</blockedMemberNode>
        <blockedMemberNode>blockedMemberNode1</blockedMemberNode>
    </replicationPolicy>
    <obsoletes>obsoletes0</obsoletes>
    <obsoletedBy>obsoletedBy0</obsoletedBy>
    <dateUploaded>2006-05-04T18:13:51.0Z</dateUploaded>
    <dateSysMetadataModified>2006-05-04T18:13:51.0Z</dateSysMetadataModified>
    <originMemberNode>originMemberNode0</originMemberNode>
    <authoritativeMemberNode>authoritativeMemberNode0</authoritativeMemberNode>
    <replica>
        <replicaMemberNode>replicaMemberNode0</replicaMemberNode>
        <replicationStatus>queued</replicationStatus>
        <replicaVerified>2006-05-04T18:13:51.0Z</replicaVerified>
    </replica>
    <replica>
        <replicaMemberNode>replicaMemberNode1</replicaMemberNode>
        <replicationStatus>queued</replicationStatus>
        <replicaVerified>2006-05-04T18:13:51.0Z</replicaVerified>
    </replica>
</d1:systemMetadata>
"""


binary_data = "\
\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\x0a\x0b\x0c\x0d\x0e\x0f\
\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f\
\x20\x21\x22\x23\x24\x25\x26\x27\x28\x29\x2a\x2b\x2c\x2d\x2e\x2f\
\x30\x31\x32\x33\x34\x35\x36\x37\x38\x39\x3a\x3b\x3c\x3d\x3e\x3f\
\x40\x41\x42\x43\x44\x45\x46\x47\x48\x49\x4a\x4b\x4c\x4d\x4e\x4f\
\x50\x51\x52\x53\x54\x55\x56\x57\x58\x59\x5a\x5b\x5c\x5d\x5e\x5f\
\x60\x61\x62\x63\x64\x65\x66\x67\x68\x69\x6a\x6b\x6c\x6d\x6e\x6f\
\x70\x71\x72\x73\x74\x75\x76\x77\x78\x79\x7a\x7b\x7c\x7d\x7e\x7f\
\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f\
\x90\x91\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f\
\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf\
\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf\
\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf\
\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf\
\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef\
\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff\
"

mmp_doc = "------------6B3C785C-6290-11DF-A355-A6ECDED72085_\x24\x0d\n\
Content-Disposition\x3a\x20form-data\x3b\x20name\x3d\x22field_name_1\x22\x0d\n\
\x0d\n\
field_value_1\x0d\n\
------------6B3C785C-6290-11DF-A355-A6ECDED72085_\x24\x0d\n\
Content-Disposition\x3a\x20form-data\x3b\x20name\x3d\x22field_name_2\x22\x0d\n\
\x0d\n\
field_value_2\x0d\n\
------------6B3C785C-6290-11DF-A355-A6ECDED72085_\x24\x0d\n\
Content-Disposition\x3a\x20form-data\x3b\x20name\x3d\x22field_name_3\x22\x0d\n\
\x0d\n\
field_value_3\x0d\n\
------------6B3C785C-6290-11DF-A355-A6ECDED72085_\x24\x0d\n\
Content-Disposition\x3a\x20form-data\x3b\x20name\x3d\x22file_key_1\x22\x3b\x20filename\x3d\x22file_name_1\x22\x0d\n\
Content-Type\x3a\x20application\x2foctet-stream\x0d\n\
\x0d\n\
\x3c\x3fxml\x20version\x3d\x221\x2e0\x22\x20encoding\x3d\x22UTF-8\x22\x3f\x3e\n\
\x3cd1\x3asystemMetadata\x20xmlns\x3ad1\x3d\x22http\x3a\x2f\x2fns\x2edataone\x2eorg\x2fservice\x2ftypes\x2fv1\x22\n\
\x20xmlns\x3axsi\x3d\x22http\x3a\x2f\x2fwww\x2ew3\x2eorg\x2f2001\x2fXMLSchema-instance\x22\n\
\x20xsi\x3aschemaLocation\x3d\x22http\x3a\x2f\x2fns\x2edataone\x2eorg\x2fservice\x2ftypes\x2fv1\x22\x3e\n\
\x20\x20\x20\x20\x3cidentifier\x3eidentifier0\x3c\x2fidentifier\x3e\n\
\x20\x20\x20\x20\x3cobjectFormat\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3cformatId\x3eformatId0\x3c\x2fformatId\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3cformatName\x3eformatName0\x3c\x2fformatName\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3cscienceMetadata\x3efalse\x3c\x2fscienceMetadata\x3e\n\
\x20\x20\x20\x20\x3c\x2fobjectFormat\x3e\n\
\x20\x20\x20\x20\x3csize\x3e-1073741773\x3c\x2fsize\x3e\n\
\x20\x20\x20\x20\x3cchecksum\x20algorithm\x3d\x22SHA-1\x22\x3echecksum0\x3c\x2fchecksum\x3e\n\
\x20\x20\x20\x20\x3csubmitter\x3esubmitter0\x3c\x2fsubmitter\x3e\n\
\x20\x20\x20\x20\x3crightsHolder\x3erightsHolder0\x3c\x2frightsHolder\x3e\n\
\x20\x20\x20\x20\x3caccessPolicy\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3callow\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x3csubject\x3esubject0\x3c\x2fsubject\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x3csubject\x3esubject1\x3c\x2fsubject\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x3cpermission\x3eread\x3c\x2fpermission\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x3cpermission\x3eread\x3c\x2fpermission\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3c\x2fallow\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3callow\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x3csubject\x3esubject2\x3c\x2fsubject\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x3csubject\x3esubject3\x3c\x2fsubject\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x3cpermission\x3eread\x3c\x2fpermission\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x20\x3cpermission\x3eread\x3c\x2fpermission\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3c\x2fallow\x3e\n\
\x20\x20\x20\x20\x3c\x2faccessPolicy\x3e\n\
\x20\x20\x20\x20\x3creplicationPolicy\x20replicationAllowed\x3d\x22false\x22\x20numberReplicas\x3d\x220\x22\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3cpreferredMemberNode\x3epreferredMemberNode0\x3c\x2fpreferredMemberNode\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3cpreferredMemberNode\x3epreferredMemberNode1\x3c\x2fpreferredMemberNode\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3cblockedMemberNode\x3eblockedMemberNode0\x3c\x2fblockedMemberNode\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3cblockedMemberNode\x3eblockedMemberNode1\x3c\x2fblockedMemberNode\x3e\n\
\x20\x20\x20\x20\x3c\x2freplicationPolicy\x3e\n\
\x20\x20\x20\x20\x3cobsoletes\x3eobsoletes0\x3c\x2fobsoletes\x3e\n\
\x20\x20\x20\x20\x3cobsoletedBy\x3eobsoletedBy0\x3c\x2fobsoletedBy\x3e\n\
\x20\x20\x20\x20\x3cdateUploaded\x3e2006-05-04T18\x3a13\x3a51\x2e0Z\x3c\x2fdateUploaded\x3e\n\
\x20\x20\x20\x20\x3cdateSysMetadataModified\x3e2006-05-04T18\x3a13\x3a51\x2e0Z\x3c\x2fdateSysMetadataModified\x3e\n\
\x20\x20\x20\x20\x3coriginMemberNode\x3eoriginMemberNode0\x3c\x2foriginMemberNode\x3e\n\
\x20\x20\x20\x20\x3cauthoritativeMemberNode\x3eauthoritativeMemberNode0\x3c\x2fauthoritativeMemberNode\x3e\n\
\x20\x20\x20\x20\x3creplica\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3creplicaMemberNode\x3ereplicaMemberNode0\x3c\x2freplicaMemberNode\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3creplicationStatus\x3equeued\x3c\x2freplicationStatus\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3creplicaVerified\x3e2006-05-04T18\x3a13\x3a51\x2e0Z\x3c\x2freplicaVerified\x3e\n\
\x20\x20\x20\x20\x3c\x2freplica\x3e\n\
\x20\x20\x20\x20\x3creplica\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3creplicaMemberNode\x3ereplicaMemberNode1\x3c\x2freplicaMemberNode\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3creplicationStatus\x3equeued\x3c\x2freplicationStatus\x3e\n\
\x20\x20\x20\x20\x20\x20\x20\x20\x3creplicaVerified\x3e2006-05-04T18\x3a13\x3a51\x2e0Z\x3c\x2freplicaVerified\x3e\n\
\x20\x20\x20\x20\x3c\x2freplica\x3e\n\
\x3c\x2fd1\x3asystemMetadata\x3e\n\
\x0d\n\
------------6B3C785C-6290-11DF-A355-A6ECDED72085_\x24\x0d\n\
Content-Disposition\x3a\x20form-data\x3b\x20name\x3d\x22file_key_2\x22\x3b\x20filename\x3d\x22file_name_2\x22\x0d\n\
Content-Type\x3a\x20application\x2foctet-stream\x0d\n\
\x0d\n\
\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\n\
\x0b\x0c\x0d\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f\x20\x21\x22\x23\x24\x25\x26\x27\x28\x29\x2a\x2b\x2c-\x2e\x2f0123456789\x3a\x3b\x3c\x3d\x3e\x3f\x40ABCDEFGHIJKLMNOPQRSTUVWXYZ\x5b\x5c\x5d\x5e_\x60abcdefghijklmnopqrstuvwxyz\x7b\x7c\x7d\x7e\x7f\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f\x90\x91\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff\x0d\n\
------------6B3C785C-6290-11DF-A355-A6ECDED72085_\x24\x0d\n\
Content-Disposition\x3a\x20form-data\x3b\x20name\x3d\x22file_key_3\x22\x3b\x20filename\x3d\x22file_name_3\x22\x0d\n\
Content-Type\x3a\x20application\x2foctet-stream\x0d\n\
\x0d\n\
\x00\x01\x02\x03\x04\x05\x06\x07\x08\x09\n\
\x0b\x0c\x0d\x0e\x0f\x10\x11\x12\x13\x14\x15\x16\x17\x18\x19\x1a\x1b\x1c\x1d\x1e\x1f\x20\x21\x22\x23\x24\x25\x26\x27\x28\x29\x2a\x2b\x2c-\x2e\x2f0123456789\x3a\x3b\x3c\x3d\x3e\x3f\x40ABCDEFGHIJKLMNOPQRSTUVWXYZ\x5b\x5c\x5d\x5e_\x60abcdefghijklmnopqrstuvwxyz\x7b\x7c\x7d\x7e\x7f\x80\x81\x82\x83\x84\x85\x86\x87\x88\x89\x8a\x8b\x8c\x8d\x8e\x8f\x90\x91\x92\x93\x94\x95\x96\x97\x98\x99\x9a\x9b\x9c\x9d\x9e\x9f\xa0\xa1\xa2\xa3\xa4\xa5\xa6\xa7\xa8\xa9\xaa\xab\xac\xad\xae\xaf\xb0\xb1\xb2\xb3\xb4\xb5\xb6\xb7\xb8\xb9\xba\xbb\xbc\xbd\xbe\xbf\xc0\xc1\xc2\xc3\xc4\xc5\xc6\xc7\xc8\xc9\xca\xcb\xcc\xcd\xce\xcf\xd0\xd1\xd2\xd3\xd4\xd5\xd6\xd7\xd8\xd9\xda\xdb\xdc\xdd\xde\xdf\xe0\xe1\xe2\xe3\xe4\xe5\xe6\xe7\xe8\xe9\xea\xeb\xec\xed\xee\xef\xf0\xf1\xf2\xf3\xf4\xf5\xf6\xf7\xf8\xf9\xfa\xfb\xfc\xfd\xfe\xff\x0d\n\
------------6B3C785C-6290-11DF-A355-A6ECDED72085_\x24\x0d\n\
Content-Disposition\x3a\x20form-data\x3b\x20name\x3d\x22file_key_4\x22\x3b\x20filename\x3d\x22file_name_4\x22\x0d\n\
Content-Type\x3a\x20application\x2foctet-stream\x0d\n\
\x0d\n\
short_data\x0d\n\
------------6B3C785C-6290-11DF-A355-A6ECDED72085_\x24--\x0d\n\
"

class TestMimeMultipart(unittest.TestCase):
  def _convert_bin_to_string_literal(self, flo):
    '''Convert a binary file to a Python string literal.
    '''
    o = open('d1_testdocs/mmp_string_literal.txt', 'wb')
    while True:
      q = flo.read(1)
      if q == '':
        break
      if q == '\n':
        o.write('\\n\\\n')
      elif re.match(r'[\w\n-]', q):
        o.write(q),
      else:
        o.write('\\x%02x' % ord(q))


  def test_correct_mmp_generation(self):
    fields = [
      ('field_name_1', 'field_value_1'),
      ('field_name_2', 'field_value_2'),
      ('field_name_3', 'field_value_3'),
    ]

    files = [
      ('file_key_1', 'file_name_1', sysmeta_xml_correct.encode('utf-8')),
      ('file_key_2', 'file_name_2', binary_data),
      ('file_key_3', 'file_name_3', StringIO.StringIO(binary_data)),
      ('file_key_4', 'file_name_4', 'short_data'),
    ]

    mmp = d1_common.mime_multipart.multipart(fields, files)

    # Create new string literal.
    #self._convert_bin_to_string_literal(mmp)
    #return

    self.assertEquals(mmp.read(), mmp_doc)

#===============================================================================

if __name__ == "__main__":
  argv = sys.argv
  if "--debug" in argv:
    logging.basicConfig(level=logging.DEBUG)
    argv.remove("--debug")
  if "--with-xunit" in argv:
    argv.remove("--with-xunit")
    unittest.main(argv=argv, testRunner=xmlrunner.XmlTestRunner(sys.stdout))
  else:
    unittest.main(argv=argv)
